﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Windows.Input;

namespace Helpers {
   [DebuggerStepThrough]
   internal class DelegateCommand : DelegateCommand<object> {
      public DelegateCommand(Action execute)
         : base((p) => execute(), null) {
      }
      public DelegateCommand(Action execute, Func<bool> canExecute)
         : base((p) => execute(), null) {
      }
   }

   [DebuggerStepThrough]
   public class DelegateCommand<T> : ICommand {
      private Func<T, bool> canExecute;
      private Action<T> execute;

      public event EventHandler CanExecuteChanged {
         add { CommandManager.RequerySuggested += value; }
         remove { CommandManager.RequerySuggested -= value; }
      }

      public DelegateCommand(Action<T> execute) : this(execute, null) { }

      public DelegateCommand(Action<T> execute, Func<T, bool> canExecute) {
         if (execute == null) throw new ArgumentNullException();
         this.execute = execute;
         this.canExecute = canExecute ?? (p => true);
      }

      public bool CanExecute(object parameter) { return this.canExecute((T)parameter); }

      public void Execute(object parameter) { if (this.CanExecute(parameter)) { this.execute((T)parameter); } }

   }
}